use axum::{
  http::{
    header::{AUTHORIZATION, WWW_AUTHENTICATE},
    Request,
  },
  middleware::Next,
  response::Response,
};
use base64::{prelude::BASE64_STANDARD, Engine};

pub async fn http_basic_auth<B>(
  request: Request<B>,
  next: Next<B>,
) -> Response {
  // do something with `request`...
  let headers = request.headers();
  let auth_fail_response = Response::builder()
    .status(401)
    .header(WWW_AUTHENTICATE, "Basic")
    .body(axum::body::boxed(String::from("")))
    .unwrap();

  let authorization = match headers.get(AUTHORIZATION) {
    Some(v) => v.to_str().unwrap(),
    None => return auth_fail_response,
  };
  let parts = authorization.split_ascii_whitespace().collect::<Vec<_>>();
  let base64part = match &parts[..] {
    ["Basic", base64part] => *base64part,
    _ => return auth_fail_response,
  };
  let decoded = match BASE64_STANDARD
    .decode(base64part.as_bytes())
    .map_err(|e| anyhow::Error::from(e))
    .and_then(|e| String::from_utf8(e).map_err(|e| e.into()))
  {
    Ok(v) => v,
    Err(_) => return auth_fail_response,
  };
  let parts = decoded.split(":").collect::<Vec<_>>();
  let (user, pass) = match &parts[..] {
    [user, pass] => (*user, *pass),
    _ => return auth_fail_response,
  };
  if !(user == "root" && pass == "hellosu") {
    return auth_fail_response;
  }

  let response = next.run(request).await;

  response
}
