use std::{pin::Pin, sync::Arc};

use futures::Future;
use tokio::sync::{
  mpsc::{self, UnboundedReceiver, UnboundedSender},
  RwLock,
};

pub type TaskQueue = deadqueue::limited::Queue<Job>;
const WORKER_COUNT: usize = 4;

pub struct Job {
  pub code: Pin<Box<dyn Future<Output = ()> + Send>>,
  pub schedule: Schedule,
}

pub enum Schedule {
  ASAP,
}

pub type JobSender = UnboundedSender<Job>;

pub struct Scheduler {
  rx: Arc<RwLock<UnboundedReceiver<Job>>>,
  queue: Arc<TaskQueue>,
}

impl Scheduler {
  pub fn new() -> (Self, Arc<TaskQueue>) {
    let (_tx, rx) = mpsc::unbounded_channel();
    let rx = Arc::new(RwLock::new(rx));
    let queue = Arc::new(TaskQueue::new(100));
    let scheduler = Scheduler {
      rx,
      queue: queue.clone(),
    };
    (scheduler, queue.clone())
  }

  pub async fn run(self) {
    for worker in 0..WORKER_COUNT {
      let queue = self.queue.clone();
      let handle = tokio::spawn(async move {
        loop {
          let task = queue.pop().await;
          println!("worker[{}] processing task ...", worker);

          task.code.await;
          println!("worker[{}] done", worker);
        }
      });
      println!("Spawned worker thread [{}]: {handle:?}", worker);
    }

    /*
    tokio::spawn(async {
      loop {
        // TODO: Implement
        // wait 30 seconds
        // query docker container status
        // add new incomplete containers to the worker queue
      }
    });
     */
  }
}
