use aah::{
  auth::http_basic_auth, jobs::Scheduler, upload::handle_upload_configuration,
  AppState,
};
use anyhow::Result;
use axum::{
  middleware::from_fn,
  response::Html,
  routing::{get, post},
  Router,
};
use futures::try_join;
use migration::{Migrator, MigratorTrait};

#[tokio::main]
async fn main() -> Result<()> {
  console_subscriber::init();

  let options = "sqlite:test.db";
  let conn = sea_orm::Database::connect(options).await?;
  Migrator::up(&conn, None).await?;

  // Run a web server
  let (sched, queue) = Scheduler::new();
  let sched_handle = tokio::spawn(sched.run());
  let state = AppState { conn, queue };
  let app = Router::new()
    .route(
      "/",
      get(|| async {
        Html::from("<form action='/upload' method=POST><input type='text' name='name' /><textarea name='config'></textarea><input type='submit' /></form>")
      }),
    )
    .route("/service/list", get(aah::api::service::list))
    .route("/service/:id", get(aah::api::service::get))
    .route("/upload", post(handle_upload_configuration))
    .layer(from_fn(http_basic_auth))
    .with_state(state);
  let webserver = axum::Server::bind(&"0.0.0.0:4000".parse().unwrap())
    .serve(app.into_make_service());
  let webserver_handle = tokio::spawn(webserver);

  let (a, _) = try_join!(webserver_handle, sched_handle).unwrap();
  a.unwrap();
  Ok(())
}
